package consume

import (
	"errors"
	"query-service/internal/errorhandler"
	"query-service/pkg/logger"
)

// Service wraps consumer methods
type Service struct {
	broker brokerdriver.Broker
}

// NewService creates a new service
func NewService(broker brokerdriver.Broker) *Service {
	return &Service{
		broker: broker,
	}
}

// Start starts consuming
func (s *Service) Start() {
	// Create consumer
	consumer := s.broker.CreateConsumer()

	consumer.SetMessageHandler(s.HandleMessage)

	// Start consuming messages
	consumer.ConsumeMessages()
}

func (s *Service) HandleMessage(msg *brokeradapter.Message) {
	// Grab sessionID from the headers
	sessionID, ok := msg.Headers["sessionID"].(string)
	if !ok {
		// TODO: Handle error where there is no session ID supplied
	}

	// Look at the result type in the headers
	switch msg.Headers["type"] {
	case "schemaResult": // A schema result
		logger.Log("Received schema result for session " + sessionID)
		// Send the result into the websocket
		s.socketService.WriteToClient(&sessionID, msg.Body)
	case "queryResult": // A query result
		logger.Log("Received query result for session " + sessionID)
		// Send the result into the websocket
		s.socketService.WriteToClient(&sessionID, msg.Body)
	default:
		logger.Log("Received untyped message for " + sessionID)
		errorhandler.LogError(errors.New("undefined message type sent"), "error while consuming")
	}
}

// // ConsumeMessageFunc is a function type to be called when a message is consumed
// type ConsumeMessageFunc func(amqp.Delivery)

// // StartConsumer will start a consumer
// // When a message is received the consumeMessage function will be called
// func StartConsumer(broker *alice.RabbitBroker, exchangeID *string, routingKey *string, consumeMessage ConsumeMessageFunc) {
// 	// Declare the exchange we want to bind to
// 	exchange, err := alice.CreateDefaultExchange(*exchangeID, alice.Direct)
// 	if err != nil {
// 		errorhandler.FailWithError(err, "failed to create exchange")
// 	}

// 	// Declare the queue we will consume from
// 	queue := alice.CreateQueue(exchange, "aql-query-queue", true, false, true, false, nil)

// 	// Create the consumer
// 	c, err := broker.CreateConsumer(queue, *routingKey, alice.DefaultConsumerErrorHandler)
// 	if err != nil {
// 		errorhandler.FailWithError(err, "failed to create consumer")
// 	}

// 	// Start consuming messages
// 	go c.ConsumeMessages(nil, false, func(msg amqp.Delivery) { consumeMessage(msg) })
// }
